;******************************
;* Siekmanski's LCFcalculator *
;******************************

.486
.model flat, stdcall  ;32 bit memory model
option casemap :none  ;case sensitive

include		windows.inc
include		kernel32.inc
include		user32.inc
include 	Comctl32.inc
include		msvcrt.inc

includelib	kernel32.lib
includelib	user32.lib
includelib	msvcrt.lib
includelib	Comctl32.lib

FLT4 MACRO float_number:REQ
LOCAL float_num
	.data
	 align 4
	 float_num real4 float_number
	.code
	EXITM <float_num>
ENDM

FLT8 MACRO float_number:REQ
LOCAL float_num
	.data
	 align 8
	 float_num real8 float_number
	.code
	EXITM <float_num>
ENDM

TEXT_ MACRO your_text:VARARG
	LOCAL text_string
	.data
	 text_string db your_text,0
	.code
	EXITM <addr text_string>
ENDM

.const

IDD_DIALOG1 			equ  101
IDC_INDUCTANCE 			equ	1002
IDC_CAPACITANCE 		equ 1003
IDC_BTN_FREQUENCY 		equ 1004
IDC_FREQUENCY 			equ 1005
IDC_BTN_CAPACITANCE		equ 1006
IDC_BTN_INDUCTANCE		equ 1007
IDC_BTN_INDUCTANCE2		equ	1008
IDC_INDUCTANCE2			equ 1009
IDC_COILLENGTH			equ 1010
IDC_COILDIAMETER		equ 1011
IDC_WIREDIAMETER		equ	1012
IDC_BTN_TURNS			equ 1013
IDC_TURNS				equ 1014
IDC_WIRE1				equ 1015
IDC_WIRE2				equ	1016
IDC_PICTURE				equ 1017
IDC_CHK1				equ 1018
IDC_RBN1 				equ 1019
IDC_RBN2 				equ	1020
IDC_INFO 				equ	1021

.data?
	align 4
hInstance				dd ?
hInfo					dd ?
hWnd					dd ?
Copy_LC					dd ?
Copy_value				dd ? ;TRUE = Cap_value
Cap_value				dd ?
Freq_value				dd ?

	align 16
Inductance_value		real8 ?
Capacitance_value		real8 ?
Frequency_value			real8 ?
Inductance_value_coil	real8 ?
Length_value			real8 ?
Diameter_value			real8 ?
WireDiameter_value		real8 ?
Wire1_value				real8 ?
Wire2_value				real8 ?
Turns_value				real8 ?
Wire_inductance			real8 ?

szDigit_buffer 			db 32 dup (?)
szString_buffer			db 512 dup (?)

.data


InfoText				db 13,10,"--- Siekmanski's Air Coil & Resonance Frequency Calculator. ---",13,10,13,10
						db "Minimum input for the air coil:",13,10,13,10
						db "a) Inductance & coil length, in order to determine the turn count.",13,10
						db "or:",13,10
						db "b) Turn count and the coil length in order to determine inductance.",13,10,13,10
						db "CAUTION! With only those inputs, a wire diameter of 0.5 mm will be used.",13,10,13,10
						db "Also, the inductance of the 2 connection wires can be calculated as you wish.",13,10,13,10
						db 'If you want to copy the data to the LC circuit, check "To LC circuit" to.',13,10,13,10
						db "You also have the choice to copy the capacitance (if frequency input field is empty, 98 MHz will be set)",13,10
						db "or frequency (if capacitance input field is empty, 40 pF will be set) to calculate LC circuit.",13,10,13,10
						db "The LC circuit must have at least 2 of the 3 input fields filled to calculate the third.",13,10,13,10
						db "This program may be used by anyone on their own responsibility.",13,10,13,10
						db "For questions or comments you may feel free to email me: siekmanski@home.nl",13,10,13,10
						db "Greetings Siekmanski "
						db 0

.code

Get_Digit proc uses ecx esi edx Digit:dword
LOCAL divisor,value:DWORD
	xor		eax,eax
	mov		value,eax
	lea		esi,szDigit_buffer
	invoke	lstrlen,esi
	mov		ecx,eax
	mov		edx,1
	mov		divisor,edx
	test	eax,eax
	jz		done
	dec		eax
	add		esi,eax
lp:	movzx	eax,byte ptr [esi]
	cmp		al,","
	jne		no_comma
	jmp		decimal
no_comma:
	cmp		al,"."
	jne		no_point
decimal:
	mov		divisor,edx
	jmp		next
no_point:	
	sub		eax,48
	imul	eax,edx
	add		value,eax
	imul	edx,10
next:
	dec		esi
	dec		ecx
	jnz		lp
	mov		eax,value
done:
	mov		esi,Digit
	fild	value			
	fidiv	divisor
	fstp	real8 ptr [esi]
	ret
Get_Digit endp

;inductance =  1 / (4 * (PI*PI) * (freq*freq) * capacitance)
;capacitance = 1 / (4 * (PI*PI) * (freq*freq) * inductance)
;freq = 1 / (2*PI*(sqrt (inductance*capacitance)))
Calculate_inductance proc Frequency:REAL8,Capacitance:REAL8,Result:DWORD
	fld 	Frequency
	fmul	FLT8(1000000.0)			; convert to MHz
	fmul	st(0),st(0)
	fld		Capacitance
	fmul	FLT8(0.000000000001)	; convert to pico Farad
	fmulp	st(1),st(0)
	fldpi
	fmul	st(0),st(0)
	fmulp	st(1),st(0)
	fmul	FLT8(4.0)
	fld1
	fdivrp	st(1),st(0)
	fmul	FLT8(1000000.0) 		; convert to micro Henry
	mov		eax,Result
	fstp	real8 ptr [eax]	
	ret
Calculate_inductance endp

Calculate_capacitance proc Frequency:REAL8,Inductance:REAL8,Result:DWORD
	fld 	Frequency
	fmul	FLT8(1000000.0)			; convert to MHz
	fmul	st(0),st(0)
	fld		Inductance
	fmul	FLT8(0.000001)	 		; convert to micro Henry
	fmulp	st(1),st(0)
	fldpi
	fmul	st(0),st(0)
	fmulp	st(1),st(0)
	fmul	FLT8(4.0)
	fld1
	fdivrp	st(1),st(0)
	fmul	FLT8(1000000000000.0)	; convert to pico Farad
	mov		eax,Result
	fstp	real8 ptr [eax]	
	ret
Calculate_capacitance endp

Calculate_frequency proc Inductance:REAL8,Capacitance:REAL8,Result:DWORD
	fld		Capacitance
	fmul	FLT8(0.000000000001)	; convert to pico Farad
	fld 	Inductance
	fmul	FLT8(0.000001)	 		; convert to micro Henry
	fmulp	st(1),st(0)
	fsqrt
	fmul	FLT8(6.283185307179586232)	; 2 * PI
	fld1
	fdivrp	st(1),st(0)
	fmul	FLT8(0.000001)			; convert to MHz
	mov		eax,Result
	fstp	real8 ptr [eax]	
	ret
Calculate_frequency endp


Calculate_inductance_LC_circuit proc

	invoke	GetDlgItemText,hWnd,IDC_FREQUENCY,addr szDigit_buffer,12
	invoke	Get_Digit,addr Frequency_value
	test	eax,eax
	jz		output_text
	invoke	GetDlgItemText,hWnd,IDC_CAPACITANCE,addr szDigit_buffer,12
	invoke	Get_Digit,addr Capacitance_value
	test	eax,eax
	jz		output_text

	invoke	Calculate_inductance,Frequency_value,Capacitance_value,addr Inductance_value
	mov		eax,"o.k."
output_text:
	.if (eax)
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Inductance_value
		invoke	SetDlgItemText,hWnd,IDC_INDUCTANCE,addr szString_buffer			
	.else
		invoke	SetDlgItemText,hWnd,IDC_INDUCTANCE,TEXT_("Input error ! ")			
	.endif
	ret
Calculate_inductance_LC_circuit endp


Calculate_capacitance_LC_circuit proc
	invoke	GetDlgItemText,hWnd,IDC_FREQUENCY,addr szDigit_buffer,12
	invoke	Get_Digit,addr Frequency_value
	test	eax,eax
	jz		output_text
	invoke	GetDlgItemText,hWnd,IDC_INDUCTANCE,addr szDigit_buffer,12
	invoke	Get_Digit,addr Inductance_value
	test	eax,eax
	jz		output_text
	invoke	Calculate_capacitance,Frequency_value,Inductance_value,addr Capacitance_value
	mov		eax,"o.k."
output_text:
	.if (eax)
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Capacitance_value
		invoke	SetDlgItemText,hWnd,IDC_CAPACITANCE,addr szString_buffer			
	.else
		invoke	SetDlgItemText,hWnd,IDC_CAPACITANCE,TEXT_("Input error ! ")			
	.endif
	ret
Calculate_capacitance_LC_circuit endp

Calculate_frequency_LC_circuit proc
	invoke	GetDlgItemText,hWnd,IDC_INDUCTANCE,addr szDigit_buffer,12
	invoke	Get_Digit,addr Inductance_value
	test	eax,eax
	jz		output_text
	invoke	GetDlgItemText,hWnd,IDC_CAPACITANCE,addr szDigit_buffer,12
	invoke	Get_Digit,addr Capacitance_value
	test	eax,eax
	jz		output_text

	invoke	Calculate_frequency,Inductance_value,Capacitance_value,addr Frequency_value
	mov		eax,"o.k."
output_text:
	.if (eax)
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Frequency_value
		invoke	SetDlgItemText,hWnd,IDC_FREQUENCY,addr szString_buffer			
	.else
		invoke	SetDlgItemText,hWnd,IDC_FREQUENCY,TEXT_("Input error ! ")			
	.endif
	ret
Calculate_frequency_LC_circuit endp


; S = CoilDiameter + WireDiameter / 2
; W = Turns
; L = Length Coil
; I = SelfInductance

; I = (S*S*W*W) / (((S*9)+(L*10))*25.4)
; W = sqrt((((S*9)+(L*10))*I*25.4) / (S*S))


;L = 210-7 h [ ln(2h/r) - 1 + r/h ] ;h = length, r = WireDiameter / 2 ;(formula in Meters and Henry's!).


Calculate_connection_wire_self_inductance proc WireDiameter:REAL8,Wire1:REAL8,Wire2:REAL8
LOCAL	wirediameterhalf,wirelength:real8

	mov		ecx,1
	lea		edx,Wire1
	fld		Wire1
	fstp	wirelength

	fld		WireDiameter
	fmul	FLT8(0.5)
	fstp	wirediameterhalf
	
lp: lea		edx,wirelength
	mov		eax,[edx]
	sub		eax,[edx+4]
	jnz		wireOK
	fldz
	fld		Wire2
	fstp	wirelength

	dec		ecx
	jz		lp
	jmp		next

wireOK:
	fld		wirelength
	fmul	FLT8(0.0002)
	
	fld		wirediameterhalf	
	fdiv	wirelength

	fld		wirelength
	fadd	st(0),st(0)
	fdiv	wirediameterhalf
	fldln2
    fxch
    fyl2x					;logN
    fsub	FLT8(1.0)
    faddp
    fmulp
	fld		Wire2
	fstp	wirelength

	dec		ecx
	jz		lp
   
next:
    faddp
    fstp	Wire_inductance
    ret
Calculate_connection_wire_self_inductance endp


Calculate_coil_self_inductance proc Turns:REAL8,CoilLength:REAL8,Diameter:REAL8,WireDiameter:REAL8,Wire1:REAL8,Wire2:REAL8,Result:DWORD
LOCAL	wirelength:real8

	invoke	Calculate_connection_wire_self_inductance,WireDiameter,Wire1,Wire2

	fld		Diameter
	fadd	WireDiameter
	fmul	FLT8(0.5)
	fst		st(1)
	fmul	st(0),st(0)
	fmul	Turns
	fmul	Turns
	fld		st(1)		
	fmul	FLT8(9.0)
	fld		CoilLength
	fmul	FLT8(10.0)
	faddp
	fmul	FLT8(25.4)
	fdivp	st(1),st(0)

	fadd	Wire_inductance

	mov		eax,Result
	fstp	real8 ptr [eax]					
	ret
Calculate_coil_self_inductance endp


Calculate_coil_turns proc Inductance:REAL8,CoilLength:REAL8,Diameter:REAL8,WireDiameter:REAL8,Wire1:REAL8,Wire2:REAL8,Result:DWORD
LOCAL	wirelength:real8

	invoke	Calculate_connection_wire_self_inductance,WireDiameter,Wire1,Wire2

	fld		Inductance
	fsub	Wire_inductance
	fstp	Inductance
	fld		Diameter
	fadd	WireDiameter
	fmul	FLT8(0.5)
	fst		st(1)
	fmul	FLT8(9.0)
	fld		CoilLength
	fmul	FLT8(10.0)
	faddp
	fmul	Inductance
	fmul	FLT8(25.4)
	fld		st(1)		
	fmul	st(0),st(0)
	fdivp	st(1),st(0)
	fsqrt
	mov		eax,Result
	fstp	real8 ptr [eax]		
	ret
Calculate_coil_turns endp


To_LC_circuit proc

	invoke	SendDlgItemMessage,hWnd,IDC_CHK1,BM_GETCHECK,0,0
	.if eax == BST_CHECKED
		fld	Inductance_value_coil
		fstp Inductance_value
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Inductance_value
		invoke	SetDlgItemText,hWnd,IDC_INDUCTANCE,addr szString_buffer			
		.if	Copy_value == TRUE
			invoke	GetDlgItemText,hWnd,IDC_FREQUENCY,addr szDigit_buffer,12
			invoke	Get_Digit,addr Frequency_value
			.if (eax==0)
				invoke	SetDlgItemText,hWnd,IDC_FREQUENCY,TEXT_("98")
			.endif
			invoke	Calculate_capacitance_LC_circuit
		.else
			invoke	GetDlgItemText,hWnd,IDC_CAPACITANCE,addr szDigit_buffer,12
			invoke	Get_Digit,addr Capacitance_value
			.if (eax==0)
				invoke	SetDlgItemText,hWnd,IDC_CAPACITANCE,TEXT_("40")		
			.endif
			invoke	Calculate_frequency_LC_circuit				
		.endif 
	.endif
	ret
To_LC_circuit endp

Check_Wire_Spacing proc
	
	fld		WireDiameter_value
	fmul	Turns_value
	fld		Length_value
	fcomp	st(1)
	fnstsw	ax
	sahf
	jae		bigger
	invoke	MessageBox,NULL,TEXT_("To many turns:",13,10,"Coil length must be larger or Wire diameter must be smaller."),TEXT_("Input error ! "),MB_OK OR MB_ICONWARNING

bigger:
	ffree	st(1)
	ffree	st(0)

	fld		WireDiameter_value
	fadd	Diameter_value
	fmul	FLT8(0.4)
	fld		Length_value
	fcomp	st(1)
	fnstsw	ax
	sahf
	jae		bigger2
	invoke	MessageBox,NULL,TEXT_("Coil diameter is too small in proportion to the length:",13,10,"Coil length should be greater than 0.4 x (Inner Coil + Wire Diameter)."),TEXT_("Input error ! "),MB_OK OR MB_ICONWARNING

bigger2:
	ffree	st(1)
	ffree	st(0)


	ret
Check_Wire_Spacing endp

Calculate_Inductance proc
	invoke	GetDlgItemText,hWnd,IDC_TURNS,addr szDigit_buffer,12
	invoke	Get_Digit,addr Turns_value
	test	eax,eax
	jz		output_text
	invoke	GetDlgItemText,hWnd,IDC_COILLENGTH,addr szDigit_buffer,12
	invoke	Get_Digit,addr Length_value
	test	eax,eax
	jz		output_text


	invoke	GetDlgItemText,hWnd,IDC_WIREDIAMETER,addr szDigit_buffer,12
	invoke	Get_Digit,addr WireDiameter_value
	test	eax,eax
	jnz		next
	fld		FLT8(0.5)
	fstp	WireDiameter_value
	invoke	SetDlgItemText,hWnd,IDC_WIREDIAMETER,TEXT_("0.5")		
next:

	invoke	GetDlgItemText,hWnd,IDC_COILDIAMETER,addr szDigit_buffer,12
	invoke	Get_Digit,addr Diameter_value
	test	eax,eax
	jnz		next2

	fld		Length_value
	fmul	FLT8(0.35)

	fstp	Diameter_value	
	invoke	sprintf,addr szString_buffer,TEXT_("%f"),Diameter_value
	invoke	SetDlgItemText,hWnd,IDC_COILDIAMETER,addr szString_buffer			
next2:

	invoke	GetDlgItemText,hWnd,IDC_WIRE1,addr szDigit_buffer,12
	invoke	Get_Digit,addr Wire1_value
	invoke	GetDlgItemText,hWnd,IDC_WIRE2,addr szDigit_buffer,12
	invoke	Get_Digit,addr Wire2_value

	invoke	Calculate_coil_self_inductance,Turns_value,Length_value,Diameter_value,WireDiameter_value,Wire1_value,Wire2_value,addr Inductance_value_coil
	mov		eax,"o.k."
output_text:
	 .if (eax)
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Inductance_value_coil

		invoke	SetDlgItemText,hWnd,IDC_INDUCTANCE2,addr szString_buffer			

		invoke	To_LC_circuit

	.else
		invoke	SetDlgItemText,hWnd,IDC_INDUCTANCE2,TEXT_("Input error ! ")
	.endif

	invoke	Check_Wire_Spacing
	ret
Calculate_Inductance endp

Calculate_Turns proc
	
	invoke	GetDlgItemText,hWnd,IDC_INDUCTANCE2,addr szDigit_buffer,12
	invoke	Get_Digit,addr Inductance_value_coil
	test	eax,eax
	jz		output_text
	invoke	GetDlgItemText,hWnd,IDC_COILLENGTH,addr szDigit_buffer,12
	invoke	Get_Digit,addr Length_value
	test	eax,eax
	jz		output_text

	invoke	GetDlgItemText,hWnd,IDC_WIREDIAMETER,addr szDigit_buffer,12
	invoke	Get_Digit,addr WireDiameter_value
	test	eax,eax
	jnz		next
	fld		FLT8(0.5)
	fstp	WireDiameter_value
	invoke	SetDlgItemText,hWnd,IDC_WIREDIAMETER,TEXT_("0.5")		
next:

	invoke	GetDlgItemText,hWnd,IDC_COILDIAMETER,addr szDigit_buffer,12
	invoke	Get_Digit,addr Diameter_value
	test	eax,eax
	jnz		next2

	fld		Length_value
	fmul	FLT8(0.35)
	fstp	Diameter_value	
	invoke	sprintf,addr szString_buffer,TEXT_("%f"),Diameter_value
	invoke	SetDlgItemText,hWnd,IDC_COILDIAMETER,addr szString_buffer			
next2:
	
	invoke	GetDlgItemText,hWnd,IDC_WIRE1,addr szDigit_buffer,12
	invoke	Get_Digit,addr Wire1_value
	invoke	GetDlgItemText,hWnd,IDC_WIRE2,addr szDigit_buffer,12
	invoke	Get_Digit,addr Wire2_value
		
	invoke	Calculate_coil_turns,Inductance_value_coil,Length_value,Diameter_value,WireDiameter_value,Wire1_value,Wire2_value,addr Turns_value
	mov		eax,"o.k."
output_text:
	 .if (eax)
		invoke	sprintf,addr szString_buffer,TEXT_("%f"),Turns_value
		invoke	SetDlgItemText,hWnd,IDC_TURNS,addr szString_buffer			

		invoke	To_LC_circuit

	.else
		invoke	SetDlgItemText,hWnd,IDC_TURNS,TEXT_("Input error ! ")
	.endif

	invoke	Check_Wire_Spacing
	ret
Calculate_Turns endp

;########################################################################

DlgProc proc hWin:HWND,uMsg:UINT,wParam:WPARAM,lParam:LPARAM

	mov		eax,uMsg
	.if eax==WM_INITDIALOG
		mov		eax,hWin
		mov		hWnd,eax

		invoke	LoadIcon,hInstance,200
		invoke	SendMessage,hWnd,WM_SETICON,1,eax	;hIcon

		invoke	SendDlgItemMessage,hWin,IDC_CHK1,BM_SETCHECK,BST_UNCHECKED,0
		invoke	SendDlgItemMessage,hWin,IDC_RBN1,BM_SETCHECK,BST_CHECKED,0
		invoke	SendDlgItemMessage,hWin,IDC_RBN2,BM_SETCHECK,BST_UNCHECKED,0

		invoke	GetDlgItem,hWin,IDC_CHK1
		mov		Copy_LC,eax
		invoke	GetDlgItem,hWin,IDC_RBN1
		mov		Cap_value,eax
		invoke	EnableWindow,eax,FALSE
		invoke	GetDlgItem,hWin,IDC_RBN2
		mov		Freq_value,eax
		invoke	EnableWindow,eax,FALSE

		mov		Copy_value,TRUE

	.elseif eax==WM_COMMAND
			mov		eax,wParam
			mov		ecx,eax
			and		eax,0ffffh
			shr		ecx,16		

			.if eax == IDC_BTN_FREQUENCY

				invoke	Calculate_frequency_LC_circuit

			.elseif eax == IDC_BTN_INDUCTANCE

				invoke	Calculate_inductance_LC_circuit

			.elseif eax == IDC_BTN_CAPACITANCE

				invoke	Calculate_capacitance_LC_circuit

			.elseif eax == IDC_BTN_TURNS
			
				invoke	Calculate_Turns

			.elseif eax == IDC_BTN_INDUCTANCE2
			
				invoke	Calculate_Inductance

			.elseif	eax == IDC_INFO
			invoke	MessageBox,NULL,addr InfoText,TEXT_("Information..."),MB_OK OR MB_ICONINFORMATION


			.elseif ecx == BN_CLICKED
				.if eax == IDC_CHK1

					invoke	SendDlgItemMessage,hWin,IDC_CHK1,BM_GETCHECK,0,0
					.if eax == BST_CHECKED
						invoke	EnableWindow,Cap_value,TRUE
						invoke	EnableWindow,Freq_value,TRUE
					.else
						invoke	EnableWindow,Cap_value,FALSE
						invoke	EnableWindow,Freq_value,FALSE
					.endif	

				.elseif eax == IDC_RBN1
						mov		Copy_value,TRUE
				.elseif eax == IDC_RBN2
						mov		Copy_value,FALSE
				.endif

			.endif

	.elseif eax==WM_CLOSE
		invoke EndDialog,hWin,0
	.else
		mov		eax,FALSE
		ret
	.endif
	mov		eax,TRUE
	ret

DlgProc endp

start:
	invoke 	GetModuleHandle,NULL
	mov		hInstance,eax
    invoke 	InitCommonControls
	invoke 	DialogBoxParam,hInstance,IDD_DIALOG1,NULL,addr DlgProc,NULL
	invoke 	ExitProcess,0
end start
